# coding: utf-8

import os
import json
from limafs import LimaFilesystem
import stat

class DummyFs(LimaFilesystem):

    def __init__(self, *args, **kwargs):
        super(DummyFs, self).__init__(*args, **kwargs)
        self.root = u'/tmp/dummyfs'
        if not os.path.exists(self.root):
            os.makedirs(self.root)

    def configure(self, port, host='127.0.0.1', mountpoint=None):
        self.mountpoint = mountpoint

    ############################################
    # Operations on metadatas
    ############################################
    def getmeta(self, path, key, default=None):
        path = os.path.join(self.root, os.path.basename(path))
        meta= u'.'.join([path, key])

        if not os.path.exists(meta):
            return default
        with open(meta, 'r') as meta:
            value = meta.read()
            return json.loads(value)

    def setmeta(self, path, key, value):
        path = os.path.join(self.root, os.path.basename(path))
        meta= u'.'.join([path, key])
        with open(meta, 'w') as meta:
            meta.write(json.dumps(value))
    def hasmeta(self, path, key):
        path = os.path.join(self.root, os.path.basename(path))
        meta= u'.'.join([path, key])
        return os.path.exists(meta)

    def persist_metas(self, path):
        pass

    ############################################
    # Operations on files and directories
    ############################################
    def exists(self, path):
        return os.path.exists(path)

    def isdir(self, path):
        return os.path.isdir(path)

    def isfile(self, st=None):
        return stat.S_ISREG(st.st_mode)

    ############################################
    # Operations on files and forks
    ############################################
    def open(self, path, mode=u'r'):
        return open(path, mode)

    def open_fork(self, path, fork, mode=u'r'):
        if fork != u'default':
            path = os.path.join(self.root, os.path.basename(path))
            fork= u'.'.join([path, fork])
        else:
            fork = path
        return self.open(fork, mode)

    def unlink_fork(self, path, fork):
        if fork != u'default':
            path = os.path.join(self.root, os.path.basename(path))
            fork= u'.'.join([path, fork])
        else:
            fork = path
        return os.unlink(fork)

    def hasfork(self, path, fork, mode=u'r'):
        if fork != u'default':
            path = os.path.join(self.root, os.path.basename(path))
            fork= u'.'.join([path, fork])
        else:
            fork = path
        return os.path.exists(fork)

    ############################################
    # Operations on directories
    ############################################
    def listdir(self, path):
        pass

    def walk(self, path):
        for p, st in self.walk_stat(path):
            yield p


    def walk_stat(self, path):
        if path != '/':
            root = os.path.join(self.mountpoint, path)
        else:
            root = self.mountpoint
        for root, dirs, files in os.walk(root):
            for f in files:
                fname = os.path.join(root, f)
                yield fname, self.stat(fname)

            for d in dirs:
                dirname = os.path.join(root, d)
                yield dirname, self.stat(dirname)
                for st, subdir in self.walk_stat(dirname):
                    yield dirname, st

    def get_system_path(self, path):
        return path

    def stat(self, path):
        return os.stat(path)

    def local(self, path=None, st=None):
        if path is not None:
            st = self.stat(path)
        return st.st_size

    def local_fork(self, path, fork):
        if fork != u'default':
            path = os.path.join(self.root, os.path.basename(path))
            fork= u'.'.join([path, fork])
        else:
            fork = path
        st = self.stat(fork)
        return self.local(st=st)

    def is_complete(self, st):
        return True

    def is_empty(self, path, fork=None):
        st = self.stat(path, fork)
        size = self.size_fork(path, fork)
        return size is None or size == 0

    def time(self, st):
        return st.st_mtime
