# coding: utf-8
from lima.meta import LimaMetaError
import abc


class LimaFsError(LimaMetaError):
    pass


class LimaFsTimeoutError(LimaFsError):
    """ Exception raised when a timeout occurs  """
    pass


class LimaFsConnectionRefusedError(LimaFsError):
    """ Exception raised when a timeout occurs  """
    pass


class LimaFsNotReadyError(LimaFsError):
    """ Exception raised when a timeout occurs  """
    pass


class LimaFsDoesnotExistError(LimaFsError):
    """ Exception raised the file/directory does not exists """
    pass


class LimaFsNotDirectoryError(LimaFsError):
    pass


class LimaFsIsDirectoryError(LimaFsError):
    pass


class LimaFsNoSpaceError(LimaFsError):
    pass


class LimaFsNotReadableError(LimaFsError):
    pass


class LimaFsNotMountedError(LimaFsError):
    pass


class LimaFsMemoryError(LimaFsError):
    pass


class LimaFsDeviceNotconfigured(LimaFsError):
    pass


class LimaFilesystem(object):
    """ Base class representing all filesystem operation used in lima-meta"""
    __metaclass__ = abc.ABCMeta

    KEY_CONTEXT = u'meta.context'

    @abc.abstractmethod
    def configure(self, *args, **kwargs):
        pass

    ############################################
    # Operations on files and directories
    ############################################
    @abc.abstractmethod
    def exists(self, path):
        pass

    @abc.abstractmethod
    def isdir(self, path, st):
        pass

    @abc.abstractmethod
    def isfile(self, st=None):
        pass

    ############################################
    # Operations on files and forks
    ############################################
    @abc.abstractmethod
    def open(self, path, mode=u'r'):
        pass

    @abc.abstractmethod
    def open_fork(self, path, mode=u'r'):
        pass

    @abc.abstractmethod
    def open_fork_meta(self, path, mode=u'r'):
        pass

    @abc.abstractmethod
    def unlink_fork(self, path, fork):
        pass

    @abc.abstractmethod
    def hasfork(self, path, fork):
        pass
    ############################################
    # Operations on directories
    ############################################

    @abc.abstractmethod
    def listdir(self, path):
        pass

    @abc.abstractmethod
    def stat(self, path):
        pass

    @abc.abstractmethod
    def walk(self, path):
        pass

    @abc.abstractmethod
    def walk_stat(self, root):
        pass

    @abc.abstractmethod
    def get_system_path(self, path):
        pass

    def getcontext(self, path):
        context = self.getmeta(path, self.KEY_CONTEXT)
        if context is None:
            context = {}
        return context

    def savecontext(self, path, context):
        if context:
            self.setmeta(path, self.KEY_CONTEXT, context)

    ###

    @abc.abstractmethod
    def local(self, st=None):
        pass

    @abc.abstractmethod
    def local_fork(self, path, fork):
        pass

    @abc.abstractmethod
    def is_empty(self, path, fork=None):
        pass

    @abc.abstractmethod
    def is_complete(self, st):
        pass

    @abc.abstractmethod
    def time(self, st):
        pass
