import os


class ErrnoException(EnvironmentError):
    ECONNREFUSED = 61
    EFATAL = 22000
    ETIMEOUT = 22003
    ENOTREADY = 22005

    # does not exists
    ENOENT = 2
    # device not configured
    ENXIO = 6

    # not enough memory
    ENOMEM = 12

    EEXIST = 17

    # Not a directory
    ENOTDIR = 20
    # is a directory
    EISDIR = 21
    # No space on device
    ENOSPC = 28
    # not readable
    ENOSTREAM = 96

    LABELS = {
        ECONNREFUSED: 'Connection refused',
        ETIMEOUT: 'Core timeout',
        ENOTREADY: 'Core not ready',
        EFATAL: 'Fatal error',
    }

    def __init__(self, err):
        super(ErrnoException, self).__init__(err, self.strerror(err))

    def strerror(self, errno):
        return self.LABELS.get(errno, os.strerror(errno))

    def is_timeout(self):
        return self.errno == ErrnoException.ETIMEOUT

    def is_conn_refused(self):
        return self.errno == ErrnoException.ECONNREFUSED

    def is_not_ready(self):
        return self.errno == ErrnoException.ENOTREADY

    def is_fatal(self):
        return self.errno == ErrnoException.EFATAL

    def is_file_does_not_exists(self):
        return self.errno == ErrnoException.ENOENT

    def is_notdir(self):
        return self.errno == ErrnoException.ENOTDIR

    def is_dir(self):
        return self.errno == ErrnoException.EISDIR

    def is_nospace(self):
        return self.errno == ErrnoException.ENOSPC

    def is_exists(self):
        return self.errno == ErrnoException.EEXIST

    def is_not_readable(self):
        return self.errno == ErrnoException.ENOSTREAM

    def is_nomemory(self):
        return self.errno == ErrnoException.ENOMEM

    def is_not_configured(self):
        return self.errno == ErrnoException.ENXIO


class ErrnoTimeoutException(ErrnoException):
    pass
