import struct
import cStringIO

class Dumper(object):
    def __init__(self, writer):
        super(Dumper, self).__init__()

        self.writer = writer

    def dump_u8(self, v):
        return self.writer(struct.pack("!B", v))

    def dump_u16(self, v):
        return self.writer(struct.pack("!H", v))

    def dump_u32(self, v):
        return self.writer(struct.pack("!I", v))

    def dump_u64(self, v):
        return self.writer(struct.pack("!Q", v))

    def dump_float(self, v):
        return self.writer(struct.pack("!f", v))

    def dump_double(self, v):
        return self.writer(struct.pack("!d", v))

    def dump_raw(self, buf):
        self.writer(buf)

    def dump_buffer(self, buf):
        self.dump_u32(len(buf))
        self.dump_raw(buf)

    dump_string = dump_buffer

    def dump(self, fmt, *kargs):
        kargs = iter(kargs)
        for i, c in enumerate(fmt):
            try:
                if c == "1":
                    self.dump_u8(kargs.next())

                elif c == "2":
                    self.dump_u16(kargs.next())

                elif c == "4":
                    self.dump_u32(kargs.next())

                elif c == "8":
                    self.dump_u64(kargs.next())

                elif c == "f":
                    self.dump_float(kargs.next())

                elif c == "d":
                    self.dump_double(kargs.next())

                elif c == "b" or c == "B":
                    self.dump_raw(kargs.next())

                elif c == "x" or c == "X":
                    self.dump_buffer(kargs.next())

                elif c == "s" or c == "S":
                    self.dump_string(kargs.next())

                else:
                    raise Exception("unknown FSON format '%s'." % (c, ))

            except Exception, e:
                raise
                #raise Exception("something's wrong with %dth format '%s': %s" % (i + 1, c, str(e), ))

        # Make sure kargs is empty
        try:
            kargs.next()

        except StopIteration:
            pass

        else:
            raise Exception("too much arguments")

class Loader(object):
    def __init__(self, reader):
        super(Loader, self).__init__()

        self.reader = reader

    def _readall(self, length):
        """ Read 'length' bytes.
            If no more data in socket, will return what was already read.
        """

        b = ""
        while len(b) < length:
            to_read = length - len(b)

            if to_read <= 0:
                break

            r = self.reader(to_read)

            if not r:
                break
                return b

            b += r

        return b

    def _load_struct(self, fmt):
        fmt_size = struct.calcsize(fmt)

        data = self._readall(fmt_size)

        if not data or len(data) < fmt_size:
            return None

        return struct.unpack(fmt, data)[0]

    def load_u8(self):
        return self._load_struct("!B")

    def load_u16(self):
        return self._load_struct("!H")

    def load_u32(self):
        return self._load_struct("!I")

    def load_u64(self):
        return self._load_struct("!Q")

    def load_float(self):
        return self._load_struct("!f")

    def load_double(self):
        return self._load_struct("!d")

    def load_raw(self, length):
        if length == 0:
            return ""

        b = self._readall(length)

        if b is None or len(b) < length:
            return

        return b

    def load_buffer(self):
        length = self.load_u32()

        if length is None:
            return

        if length == 0:
            return ""

        return self.load_raw(length)

    load_string = load_buffer

    def iload(self, fmt, *kargs):
        kargs = iter(kargs)
        for i, c in enumerate(fmt):
            try:
                if c == "1":
                    yield self.load_u8()

                elif c == "2":
                    yield self.load_u16()

                elif c == "4":
                    yield self.load_u32()

                elif c == "8":
                    yield self.load_u64()

                elif c == "f":
                    yield self.load_float()

                elif c == "d":
                    yield self.load_double()

                elif c == "b" or c == "B":
                    yield self.load_raw(kargs.next())

                elif c == "x" or c == "X":
                    yield self.load_buffer()

                elif c == "s" or c == "S":
                    yield self.load_string()

                else:
                    raise Exception("unknown FSON format '%s'." % (c, ))

            except Exception, e:
                raise
                raise Exception("something's wrong with %dth format '%s': %s" % (i + 1, c, str(e), ))


        # Make sure kargs is empty
        try:
            kargs.next()

        except StopIteration:
            pass

        else:
            raise Exception("too much arguments")

    def load(self, *kargs, **kwargs):
        return list(self.iload(*kargs, **kwargs))

class Model(object):
    def __init__(self, fmt):
        super(Model, self).__init__()

        self.fmt = fmt

    def dump(self, fp, *kargs):
        return Dumper(fp).dump(self.fmt, *kargs)

    def iload(self, fp, *kargs):
        return Loader(fp).iload(self.fmt, *kargs)

    def load(self, fp, *kargs):
        return Loader(fp).load(self.fmt, *kargs)

    def dumps(self, *kargs):
        w = cStringIO.StringIO()
        self.dump(w.write, *kargs)
        return w.getvalue()

    def loads(self, st, *kargs):
        r = cStringIO.StringIO(st)
        return Loader(r.read).load(self.fmt, *kargs)

def dump(fp, fmt, *kargs):
    return Model(fmt).dump(fp, *kargs)

def iload(fp, fmt, *kargs):
    return Model(fmt).iload(fp, *kargs)

def load(fp, fmt, *kargs):
    return Model(fmt).load(fp, *kargs)

def dumps(fmt, *kargs):
    return Model(fmt).dumps(*kargs)

def loads(s, fmt, *kargs):
    return Model(fmt).loads(s, *kargs)

u8_cast_s8   = lambda v: struct.unpack("!b", struct.pack("!B", v))[0]
s8_cast_u8   = lambda v: struct.unpack("!B", struct.pack("!b", v))[0]
u16_cast_s16 = lambda v: struct.unpack("!h", struct.pack("!H", v))[0]
s16_cast_u16 = lambda v: struct.unpack("!H", struct.pack("!h", v))[0]
u32_cast_s32 = lambda v: struct.unpack("!i", struct.pack("!I", v))[0]
s32_cast_u32 = lambda v: struct.unpack("!I", struct.pack("!i", v))[0]
u64_cast_s64 = lambda v: struct.unpack("!q", struct.pack("!Q", v))[0]
s64_cast_u64 = lambda v: struct.unpack("!Q", struct.pack("!q", v))[0]

def main():
    ARGS = (
        0x42,
        0x4242,
        0xDEADBEAF,
        0xDEADBEAFDEADBAC0,
        struct.unpack("!f", struct.pack("!f", 3.14))[0],    # 3.140000104904175
        3.1415,
        "raw_buffer",
        "hello world",
        "string"
    )

    print "input", ARGS

    m = Model("1248fdbxS")

    raw = m.dumps(
        *ARGS
    )
    print "raw", ARGS

    res = m.loads(raw, len(ARGS[6]))

    assert list(res) == list(ARGS)

    assert u8_cast_s8(0xFF) == -1
    assert u16_cast_s16(0xFFFF) == -1
    assert u32_cast_s32(0xFFFFFFFF) == -1
    assert u64_cast_s64(0xFFFFFFFFFFFFFFFF) == -1

if __name__ == "__main__":
    main()

