import fsonrpc.connection
from exc import *
import yaml
import msgpack
import os.path
import sys 

import threading

class RPCDict(dict):
    _instance = None
    lock = threading.Lock()

    def __new__(cls, yml_fh):
        with RPCDict.lock:
            if cls._instance is None:
                self = super(RPCDict, cls).__new__(cls)

                yml_fh.seek(0)
                kv = yaml.load(yml_fh)

                kv = dict((k.lower(), v) for k, v in kv.iteritems())
                self.update(kv)

                cls._instance = self

            return cls._instance


class RPCClient(object):

    def __init__(self, sock, fh_yml, timeout=None):
        super(RPCClient, self).__init__()

        self.conn = fsonrpc.connection.Connection(sock)
        self.fh_yml = fh_yml
        self.timeout = timeout

    def __getattr__(self, name):
        rpc = RPCDict(self.fh_yml).get(name.lower())

        if rpc is None:
            raise Exception("Unknown RPC %r" % (name, ))

        def wrapper(args, is_msgpack=None, no_return=None):
            is_msgpack = is_msgpack if is_msgpack is not None else True

            if is_msgpack:
                args = msgpack.packb(args, use_bin_type=True)

            try:
                value = self.conn.call(rpc["id"], args, no_return=no_return, timeout=self.timeout)

            except fsonrpc.connection.ResponseException, e:
                exc_type, excep, traceback = sys.exc_info()
                if -e.status == ErrnoTimeoutException.ETIMEOUT :
                    raise ErrnoTimeoutException, ErrnoTimeoutException(-e.status), traceback
                raise ErrnoException, ErrnoException(-e.status), traceback

            if is_msgpack:
                value = msgpack.unpackb(value, encoding="utf-8")

            return value

        wrapper.func_name = name
        return wrapper
