# coding: utf-8

import logging
logger = logging.getLogger(__name__)


class DeleteForkOnError(object):

    ''' Context that ensure that the fork opened will be unlinked
    if an exception occurs
    '''

    def __init__(self, fs, path, fork=None, mode=u'w'):
        self._fs = fs
        self._path = path
        self._fork_name = fork

        if fork is None:
            self._fork = fs.open(self._path, mode)
        else:
            self._fork = fs.open_fork(self._path, self._fork_name, mode)

    def __getattribute__(self, name):
        if name.startswith('_'):
            return object.__getattribute__(self, name)
        elif name == u"path":
            return object.__getattribute__(self, '_path')
        # we are rapping the forks
        return getattr(self._fork, name)

    def __enter__(self):
        self._fork.__enter__()
        return self

    def __exit__(self, *exc):
        exc_type, exc_inst, traceback = exc

        # close the fork
        self._fork.__exit__(*exc)

        if exc_type is not None:
            logger.debug(u'Fork will %s be unlinked', self._path)
            try:
                if self._fork_name is None:
                    self._fs.unlink(self._path)
                else:
                    # It may be the filesystem which has troube...
                    self._fs.unlink_fork(self._path, self._fork_name)
                logger.warn(u'Fork has been unlinked')
            except:
                # The FS Failed, we are over dead
                logger.exception(u'Could not unlink the fork')
            # raise original exception
            raise exc_type, exc_inst, traceback

        return True

    def __repr__(self):
        return '<DeleteForkOnError path=%s, fork=%s>' % (self.path, self._fork_name)
