# coding: utf-8
import os
import gzip
from logging.handlers import RotatingFileHandler


class GzipRotatingFileHandler(RotatingFileHandler):
    def doRollover(self):
        """
        Do a rollover, as described in __init__().
        """
        # NOTE : this code overide python2.7/logging/handlers.py:121
        # and adds gzip compression when the file has been rotated.
        # comments beginning with 'NOTE' are added by me.
        # other comments are keep as is - even commented code
        if self.stream:
            self.stream.close()
            self.stream = None
        if self.backupCount > 0:
            for i in range(self.backupCount - 1, 0, -1):
                # NOTE : line modified from original handler code
                sfn = "%s.%d.gz" % (self.baseFilename, i)
                # NOTE : line modified from original handler code
                dfn = "%s.%d.gz" % (self.baseFilename, i + 1)
                if os.path.exists(sfn):
                    # print "%s -> %s" % (sfn, dfn)
                    if os.path.exists(dfn):
                        os.remove(dfn)
                    os.rename(sfn, dfn)
            dfn = self.baseFilename + ".1"
            if os.path.exists(dfn):
                os.remove(dfn)
            # Issue 18940: A file may not have been created if delay is True.
            if os.path.exists(self.baseFilename):
                os.rename(self.baseFilename, dfn)

                # NOTE : added from original handler code
                # after that the file has been renamed, we gzip it
                with open(dfn, "rb") as sf:
                    with gzip.open(dfn + ".gz", "wb") as df:
                        df.write(sf.read())
                os.remove(dfn)

        if not self.delay:
            self.stream = self._open()
