# coding: utf-8
from lima.mimes import mimetypes

from lima.fs import FileSystem as fs
from base import BlackBoardAgent
from classes import VeryLowCPUClass

__all__ = [
    'FileTypeMixin',
    'WithForkMixin',
]


class FileTypeMixin(object):

    def _check_mime(self, meta_storage, *expected_mimes):
        mime, _ = mimetypes.guess_type(self.path)
        if mime is None:
            return False
        result = False
        for expected_mime in expected_mimes:
            result = result or mime.startswith(expected_mime)
        return result

    def _check_mime_not(self, meta_storage, *expected_mimes):
        mime, _ = mimetypes.guess_type(self.path)
        if mime is None:
            return True
        result = False
        for expected_mime in expected_mimes:
            if mime.startswith(expected_mime):
                return False
        return True


class WithForkMixin(object):

    ''' Class for agent with forks'''

    def has_any_fork(self, storage, *forks):
        for fork in forks:
            if storage.exists(fork):
                return True
        return False

    def has_all_forks(self, storage, *forks):
        for fork in forks:
            if not storage.exists(fork):
                return False
        return True

    def has_all_forks_non_empty(self, storage, *forks):
        for fork in forks:
            if not storage.exists_non_empty(fork):
                return False
        return True
