import peewee
from playhouse.migrate import *
from lima import db
from lima.models import LimaNode

import logging
logger = logging.getLogger(__name__)


lima_db = db.get_database()
migrator = SqliteMigrator(lima_db)
total_file_count = IntegerField(null=True)


def up():
    with lima_db.transaction():
        try:
            migrate(
                migrator.alter_add_column('limanode', 'total_file_count', total_file_count),
            )
        except peewee.OperationalError as e:
            if e.message == 'duplicate column name: total_file_count':
                logger.debug("Column '%s' already present", 'total_file_count')
            else:
                msg = getattr(e, 'message', "")
                logger.error("Could not add new column : %s", msg)
    with lima_db.transaction():
        try:
            LimaNode.update(
                status=LimaNode.STATUS_CREATED
            ).where(LimaNode.content_type == 'episode').execute()
        except Exception as e:
            msg = getattr(e, 'message', "")
            logger.error("Could not update episodes status : %s", msg)


def down():
    with lima_db.transaction():
        migrate(
            migrator.drop_column('total_file_count', 'total_file_count')
        )
