# coding: utf-8
"""
Re apply 004 that may have failed during the update.
exceptions are not swallowed.
"""
import peewee
from playhouse.migrate import *
from lima import db
from lima.models import LimaNode

import logging
logger = logging.getLogger(__name__)

lima_db = db.get_database()
migrator = SqliteMigrator(lima_db)
meta_version = IntegerField(null=False, default=0)
status_index = IntegerField(null=False,
                            index=True,
                            default=LimaNode.STATUS_INDEX_NONE)

# handle levels of file modification
is_content_modified = BooleanField(default=False)
is_path_modified = BooleanField(default=False)
is_basename_modified = BooleanField(default=False)
# we need to know if the file had been originally created on this device
is_local_source = BooleanField(default=False)



def up():
    with lima_db.transaction():
        try:
            migrate(migrator.alter_add_column('limanode', 'meta_version', meta_version), )
        except peewee.OperationalError as e:
            if e.message == 'duplicate column name: meta_version':
                logger.debug("Column '%s' already present", 'meta_version')
            else:
                raise

        try:
            try:
                # Drop old boolean column
                migrate(migrator.drop_column('limanode', 'need_reindex'))
            except ValueError as e:
                if "does not exist" in e.message:
                    # already removed.
                    pass
                else:
                    raise
            # replace with integer field
            migrate(migrator.alter_add_column('limanode', 'status_index', status_index), )
        except peewee.OperationalError as e:
            if e.message == 'duplicate column name: status_index':
                logger.debug("Column '%s' already present", 'status_index')
            else:
                raise

        LimaNode.update(status_index=LimaNode.STATUS_INDEX_NONE).execute()
        LimaNode.update(meta_version=0).execute()

        try:
            migrate(migrator.alter_add_column('limanode', 'is_content_modified', is_content_modified), )
        except peewee.OperationalError as e:
            if 'duplicate column name' in e.message:
                logger.debug("Column '%s' already present", 'is_content_modified')
            else:
                raise
        try:
            migrate(migrator.alter_add_column('limanode', 'is_path_modified', is_path_modified), )
        except peewee.OperationalError as e:
            if 'duplicate column name' in e.message:
                logger.debug("Column '%s' already present", 'is_path_modified')
            else:
                raise
        try:
            migrate(migrator.alter_add_column('limanode', 'is_basename_modified', is_basename_modified), )
        except peewee.OperationalError as e:
            if 'duplicate column name' in e.message:
                logger.debug("Column '%s' already present", 'is_basename_modified')
            else:
                raise


        LimaNode.update(is_content_modified=False, is_path_modified = False, is_basename_modified=False).execute()

        try:
            migrate(migrator.alter_add_column('limanode', 'is_local_source', is_local_source), )
        except peewee.OperationalError as e:
            if 'duplicate column name' in e.message:
                logger.debug("Column '%s' already present", 'meta_version')
            else:
                raise

        LimaNode.update( is_local_source=False,).execute()


def down():
    pass
