import logging

class LoggerProxy(object):
    def __init__(self, name, tag):
        self.logger = logging.getLogger(name)
        if tag:
            # remove 'dangerous' chars fron file name as it could be interpreted as logging formating
            tag = unicode(tag).replace(u'%', '').replace('{', '').replace('}', '')
            self.tag = u"["+tag+"] "
        else:
            self.tag = None

    def info(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.info(message, *args, **kwargs)

    def debug(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.debug(message, *args, **kwargs)

    def warn(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.warn(message, *args, **kwargs)

    def warning(self, *args, **kwargs):
        return self.warn(*args, **kwargs)

    def error(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.error(message, *args, **kwargs)

    def exception(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.exception(message, *args, **kwargs)

    def fatal(self, message, *args, **kwargs):
        if self.tag:
            message = self.tag + message
        return self.logger.fatal(message, *args, **kwargs)


def getLogger(name, tag=None):
    return LoggerProxy(name, tag)

