# coding: utf-8
import getpass
import analytics
import traceback

from lima.fs import FileSystem as fs
from lima.fs.fcfs import (LimaFsError, LimaFsTimeoutError,
                          LimaFsConnectionRefusedError, LimaFsNotReadyError
                          )

CONFIG = {}

import logging
logger = logging.getLogger(__name__)

DEVICES_LABELS = ['Unknown',
                  'LIMA_V1',
                  'Mac',
                  'Windows',
                  'Linux',
                  'iOS',
                  'Android', ]


def configure(prefix, api_key, version, is_debug):
    """Configure analitycs api_key"""
    analytics.write_key = api_key
    CONFIG['prefix'] = prefix
    CONFIG['version'] = version
    CONFIG['is_debug'] = is_debug


def track(*args, **kwargs):
    try:
        if CONFIG.get('user_id', None) is None or\
                CONFIG.get('devtype', None) is None:
            fs_status = fs.status
            CONFIG['user_id'] = fs_status.get("user_id", 'Unknown')
            try:
                CONFIG['device'] = DEVICES_LABELS[int(fs_status.get('devtype', 0))]
            except:
                CONFIG['device'] = DEVICES_LABELS[0]
    except LimaFsTimeoutError:
        logger.error("Core timeout")
        return
    except LimaFsConnectionRefusedError:
        logger.error("Connection refused")
        return
    except LimaFsNotReadyError:
        logger.error("Core not ready")
        return
    except LimaFsError:
        logger.exception(u"Core error")
        return

    context = {
        'version': CONFIG['version'],
        'device': CONFIG['device'],
        'debug': CONFIG['is_debug']
    }
    if 'event' not in kwargs:
        raise ValueError("event should be passed as kwarg")
    kwargs['event'] = "%s > %s" % (CONFIG['prefix'], kwargs['event'])
    try:
        return analytics.track(CONFIG['user_id'],
                               context=context, *args, **kwargs)
    except Exception:
        pass


def cleanup_backtrace(backtrace):
    """ """
    obfuscated_username = "YYY"
    current_user = getpass.getuser()
    backtrace = backtrace.replace(current_user, obfuscated_username)
    return backtrace


def track_error(*args, **kwargs):
    """Track error.
    Will insert traceback in properties
    """
    properties = kwargs.get('properties', {})
    backtrace = traceback.format_exc()
    try:
        backtrace = cleanup_backtrace(backtrace)
    except Exception:
        logger.error("Could not cleanup backtrace")

    properties['exception'] = backtrace

    kwargs['properties'] = properties
    track(*args, **kwargs)


def flush():
    analytics.flush()
