# coding: utf-8
# Hack to be able to import hreading which have the same name as this module
from __future__ import absolute_import
import sys
import threading
from threading import *
import abc
from lima.tools import segment
import os
import peewee
from lima import db

import logging
logger = logging.getLogger(__name__)


class ThreadKilledError(Exception):
    pass

class ThreadStoppedError(Exception):
    pass

class Thread(threading.Thread):

    __metaclass__ = abc.ABCMeta

    def __init__(self):
        threading.Thread.__init__(self)
        self.daemon = True

        # Thread stop event
        self._resume_event = threading.Event()
        # Thread stop event
        self._kill_event = threading.Event()

        self._exc_info = (None, None, None)

    def run(self):
        try:
            self._resume_event.set()
            self._kill_event.clear()
            self.execute()
        except peewee.DatabaseError as e:
            # this is very bad, main sqlite db is corrupted.
            if "database disk image is malformed" in e.message:
                # this is very bad, main sqlite db is corrupted.
                # only safe solution : delete the file, let the exception
                # be raised and pray for next run to work
                logger.exception(u"Database corrupted")
                segment.track_error(event="Database corrupted")
                try:
                    try:
                        db.get_database().close()
                    except Exception:
                        logger.exception(u"Could not close corrupted db")
                    os.unlink(db.get_db_path())
                except Exception:
                    logger.exception(u"Could not remove corrupted db")
                    segment.track_error(event="Database corrupted : could not remove db file")
            self._exc_info = sys.exc_info()
        except Exception:
            logger.critical(u"Caught exception in thread")
            logger.exception(u"Unhandled error")
            segment.track_error(event="Unhandled error")
            self._exc_info = sys.exc_info()

    def exc_info(self):
        """ Method similar to sys.exc_info
        Will return exception and traceback if the thread raised an error.
        """
        return self._exc_info

    @abc.abstractmethod
    def execute(self, *args, **kwargs):
        pass

    def stop(self):
        """
        Clear the resume event.
        """
        # unset the resume event, the loop will wait for this event
        self._resume_event.clear()

    def resume(self):
        """ Resume this thread.
        """
        # Set the resume event
        self._resume_event.set()
        self._kill_event.clear()

    def kill(self, fatal=False):
        """ Kill this thread.
        :param: fatal : set the exc_info attribute with current backtrace to inform the main scheduler that an error occured.
        """
        if fatal:
            self._exc_info = sys.exc_info()
        self._resume_event.clear()
        # Set the kill event
        self._kill_event.set()

    def wait(self, sec):
        self._kill_event.wait(sec)

    def check_continue(self):
        if self._kill_event.isSet():
            raise ThreadKilledError('Killed')
        if not self._resume_event.isSet():
            raise ThreadStoppedError('Stopped')

    @property
    def should_continue(self):
        """ Return True if the thread should continue his work"""
        if self._kill_event.isSet():
            return False
        return self._resume_event.isSet()

    @property
    def should_die(self):
        return self._kill_event.isSet()
