# coding: utf-8
import json
import sys
import thread
from bottle import Bottle, response, request
from lima.tools import segment

webapp = Bottle()

import logging
logger = logging.getLogger(__name__)

@webapp.hook('after_request')
def enable_cors():
    """
    You need to add some headers to each request.
    Don't use the wildcard '*' for Access-Control-Allow-Origin in production.
    """
    response.headers['Access-Control-Allow-Origin'] = 'file://'
    response.headers['Access-Control-Allow-Methods'] = 'GET, POST'
    response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type'


def lima_app():
    return webapp.config['lima_app']


@webapp.get('/app/resume')
@webapp.post('/app/resume')
@webapp.get('/app/start')
@webapp.post('/app/start')
def resume():
    # as lima app is a thread, it will already be started here.
    # we just call enable on it
    try:
        app = lima_app()
        if not app.is_running():
            params = request.json
            if params is None:
                params = {
                        k:v for k, v in request.params.items()
            }
            if len(params) == 0:
                # support old style - enabled by default
                params = {'externals': 1}
            lima_app().resume(params)
            segment.track(event="Enabled")

            response.content_type = 'application/json'
        return json.dumps({'success': True})
    except:
        logger.exception("Could not resume")
        return json.dumps({'success': False})


@webapp.get('/app/stop')
def stop():
    try:
        logger.info(u"Received stop signal")
        lima_app().stop()
        response.content_type = 'application/json'
        segment.track(event="Disabled")
        return json.dumps({'success': True})
    except:
        logger.exception("Could not stop")
        return json.dumps({'success': False})

@webapp.get('/app/kill')
def kill():
    try:
        logger.info(u"Received kill signal")
        lima_app().kill()
        response.content_type = 'application/json'
        segment.track(event="Killed")
        webapp.server.stop()
    except:
        sys.exit()

@webapp.post('/app/configure')
def configure():
    try:
        params = request.json
        if params is None:
            params = {
                    k:v for k, v in request.params.items()
        }
        if len(params) == 0:
            # support old style - enabled by default
            params = {'externals': 1}
        lima_app().update_config(params)
        response.content_type = 'application/json'
        return json.dumps({'success': True})
    except:
        logger.exception("Could not configure")
        return json.dumps({'success': False})


@webapp.get('/app/status')
def status():
    try:
        app = lima_app()
        is_running = app.is_running()
        is_processing = app.is_processing()
        has_kill_switch = app.have_killswitch

        response.content_type = 'application/json'
        return json.dumps({
            'success': True,
            'running': is_running,
            'processing': is_processing,
            'kill_switch': has_kill_switch,
        })
    except:
        logger.exception("Could not get status")
        return json.dumps({'success': False})


from bottle import ServerAdapter


class LimaWSGIRefServer(ServerAdapter):
    server = None

    def run(self, handler):
        from wsgiref.simple_server import make_server, WSGIRequestHandler
        if self.quiet:

            class QuietHandler(WSGIRequestHandler):
                def log_request(*args, **kw):
                    pass

            self.options['handler_class'] = QuietHandler
        # Hack to keep a reference on the server from the app
        # to be able to kill it from a route
        setattr(handler, 'server', self)
        self.server = make_server(self.host, self.port, handler, **self.options)
        self.server.serve_forever()

    def stop(self):
        thread.start_new_thread(self.server.shutdown, ())
